enum IndexBucket {
Empty {
ptr: *mut Index,
},
Occupied {
ptr: *mut Index,
header: DiskBlockHeader,
},
}Expand description
A bucket in our index map, holding a pointer to the index.
This enum indicates whether a given bucket is occupied, and if it is, it holds the respective block header as well.
Variants
Empty
This bucket is empty
If this is a search, this means the entry isn’t in the map, and this is where it would be
Occupied
This bucket is occupied. We can read or overwrite the index value from the pointer.