Struct HashedDiskHeader

#[repr(C)]
struct HashedDiskHeader { header: DiskBlockHeader, hash: BlockHash, acc_pos: u32, acc_len: u32, }
To avoid having to sha256 headers every time we retrieve them, we store the hash along with the header, so we just need to compare the hash to know if we have the right header

Fields

header: DiskBlockHeader

The actual header with contextually relevant information

hash: BlockHash

The hash of the header

acc_pos: u32

Where in the accumulator file this block’s accumulator is

acc_len: u32

The length of the block’s accumulator