#[repr(C)]struct HashedDiskHeader {
header: DiskBlockHeader,
hash: BlockHash,
acc_pos: u32,
acc_len: u32,
}Expand description
To avoid having to sha256 headers every time we retrieve them, we store the hash along with the header, so we just need to compare the hash to know if we have the right header
Fields
header: DiskBlockHeaderThe actual header with contextually relevant information
hash: BlockHashThe hash of the header
acc_pos: u32Where in the accumulator file this block’s accumulator is
acc_len: u32The length of the block’s accumulator